package jdk.internal.instrumentation;

import java.security.AccessController;
import java.security.Permission;
import java.security.PrivilegedAction;
import java.util.Iterator;
import sun.reflect.CallerSensitive;
import sun.misc.VM;
import sun.reflect.Reflection;
import java.util.ArrayList;
import java.util.List;

public final class Tracer
{
    private final List<Tracer.InstrumentationData> items;
    private static final Tracer singleton;
    
    private Tracer() {
        this.items = new ArrayList();
    }
    
    @CallerSensitive
    public static Tracer getInstance() {
        if (!VM.isSystemDomainLoader(Reflection.getCallerClass().getClassLoader())) {
            throw new SecurityException("Only classes in the system domain can get a Tracer instance");
        }
        return Tracer.singleton;
    }
    
    public synchronized void addInstrumentations(List<Class<?>> list, Logger logger) throws ClassNotFoundException {
        if (logger == null) {
            throw new IllegalArgumentException("logger can't be null");
        }
        ArrayList<Tracer.InstrumentationData> list2 = new ArrayList();
        for (Class instrumentation : list) {
            InstrumentationTarget instrumentationTarget = (InstrumentationTarget)instrumentation.getAnnotation(InstrumentationTarget.class);
            Tracer.InstrumentationData o = new Tracer.InstrumentationData();
            o.instrumentation = instrumentation;
            o.target = Class.forName(instrumentationTarget.value(), true, instrumentation.getClassLoader());
            o.logger = logger;
            list2.add(o.target);
            this.items.add(o);
        }
        retransformClasses0((Class[])list2.toArray(new Class[0]));
        return;
    }
    
    private byte[] transform(Class<?> obj, byte[] newBytes) {
        for (Tracer.InstrumentationData instrumentationData : this.items) {
            if (instrumentationData.target.equals(obj)) {
                try {
                    instrumentationData.logger.trace("Processing instrumentation class: " + instrumentationData.instrumentation);
                    newBytes = new ClassInstrumentation(instrumentationData.instrumentation, obj.getName(), newBytes, instrumentationData.logger).getNewBytes();
                }
                catch (Throwable t) {
                    instrumentationData.logger.error("Failure during class instrumentation:", t);
                }
            }
            else {
                continue;
            }
        }
        if (newBytes == newBytes) {
            return null;
        }
        return newBytes;
    }
    
    private static native void retransformClasses0(Class<?>[] p0);
    
    private static byte[] retransformCallback(Class<?> clazz, byte[] array) {
        return Tracer.singleton.transform(clazz, array);
    }
    
    private static native void init();
    
    static {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                System.loadLibrary("bci");
                return null;
            }
        }, null, new Permission[] { new RuntimePermission("loadLibrary.bci") });
        singleton = new Tracer();
        init();
    }
    
    private final class InstrumentationData
    {
        Class<?> instrumentation;
        Class<?> target;
        Logger logger;
    }
}
